@@ -6,7 +6,7 @@
 
 package Lemonldap::NG::Common::Session;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.9';
 
 use Lemonldap::NG::Common::Apache::Session;
 
@@ -123,14 +123,10 @@ sub BUILD {
         $data = $self->_tie_session;
     }
 
-    # If session is created
-    # Then set session kind in session
-    if ( $creation and $self->kind ) {
-        $data->{_session_kind} = $self->kind;
-    }
-
     if ( $self->{info} ) {
         foreach ( keys %{ $self->{info} } ) {
+            next if ( $_ eq "_session_id" and $data->{_session_id} );
+            next if ( $_ eq "_session_kind" and $data->{_session_kind});
             if ( defined $self->{info}->{$_} ) {
                 $data->{$_} = $self->{info}->{$_};
             }
@@ -141,6 +137,12 @@ sub BUILD {
         delete $self->{info};
     }
 
+    # If session is created
+    # Then set session kind in session
+    if ( $creation and $self->kind ) {
+        $data->{_session_kind} = $self->kind;
+    }
+
     # Load session data into object
     if ($data) {
         if ( $self->kind and $data->{_session_kind} ) {
