@@ -24,9 +24,12 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.9';
 
-extends 'Lemonldap::NG::Portal::Main::Auth', 'Lemonldap::NG::Portal::Lib::SAML';
+extends qw(
+  Lemonldap::NG::Portal::Main::Auth
+  Lemonldap::NG::Portal::Lib::SAML
+);
 
 # INTERFACE
 
@@ -778,7 +781,7 @@ sub extractFormInfo {
                 $req->response( [
                         200,
                         [
-                            'Content-Type'   => 'application/xml',
+                            'Content-Type'   => 'text/xml',
                             'Content-Length' => length($slo_body)
                         ],
                         [$slo_body]
@@ -846,7 +849,7 @@ sub extractFormInfo {
         $req->response( [
                 200,
                 [
-                    'Content-Type'   => 'application/xml',
+                    'Content-Type'   => 'text/xml',
                     'Content-Length' => length($art_response)
                 ],
                 [$art_response]
@@ -972,7 +975,7 @@ sub extractFormInfo {
               or $a->{name} cmp $b->{name}
               or $a->{val} cmp $b->{val}
           } @list;
-        $req->data->{list}            = \@list;
+        $req->data->{list} = \@list;
 
         #TODO: check this
         $req->data->{login} = 1;
@@ -1267,7 +1270,8 @@ sub authFinish {
 
     $self->logger->debug( "Store NameID "
           . $nameid->dump
-          . " and SessionIndex $session_index for session $id" );
+          . ( $session_index ? " and SessionIndex $session_index" : "" )
+          . " for session $id" );
 
     my $infos;
     $infos->{type}          = 'saml';            # Session type
@@ -1363,7 +1367,7 @@ sub authLogout {
 
         $req->urldc($slo_url);
 
-        # Redirect done in Portal/Simple.pm
+        # Redirect done in Portal
         return PE_OK;
     }
 
@@ -1381,10 +1385,10 @@ sub authLogout {
         $req->postFields( { 'SAMLRequest' => $slo_body } );
 
         # RelayState
-        $self->postFields->{'RelayState'} = $logout->msg_relayState
+        $req->postFields->{'RelayState'} = $logout->msg_relayState
           if ( $logout->msg_relayState );
 
-        # Post done in Portal/Simple.pm
+        # Post done in Portal
         $req->steps( [ 'deleteSession', 'autoPost' ] );
         return PE_OK;
     }
@@ -1472,7 +1476,8 @@ sub getIDP {
                 if ( $idpName eq $idpConfKey ) {
                     $idp = $_;
                     $self->logger->debug(
-                        "IDP $idp selected from idpName URL Parameter ($idpName)");
+"IDP $idp selected from idpName URL Parameter ($idpName)"
+                    );
                     last;
                 }
             }
@@ -1510,7 +1515,7 @@ sub getIDP {
         $idp = undef;
     }
 
-    return  $idp;
+    return $idp;
 }
 
 1;
