@@ -9,22 +9,33 @@
 #
 package Lemonldap::NG::Portal::Main::Run;
 
-our $VERSION = '2.0.7';
+our $VERSION = '2.0.9';
 
 package Lemonldap::NG::Portal::Main;
 
 use strict;
 use URI::Escape;
 use JSON;
+use Lemonldap::NG::Common::Util qw(getPSessionID);
 
 has trOverCache => ( is => 'rw', default => sub { {} } );
 
+# The execution order between groups and macros can be
+# modified in config (#1877)
+sub groupsAndMacros {
+    return (
+        $_[0]->conf->{groupsBeforeMacros}
+        ? qw(setGroups setMacros)
+        : qw(setMacros setGroups)
+    );
+}
+
 # List constants
 sub authProcess { qw(extractFormInfo getUser authenticate) }
 
 sub sessionData {
-    qw(setAuthSessionInfo setSessionInfo setMacros setGroups setPersistentSessionInfo
-      setLocalGroups store secondFactor);
+    return qw(setAuthSessionInfo setSessionInfo), $_[0]->groupsAndMacros,
+      qw(setPersistentSessionInfo setLocalGroups store secondFactor);
 }
 
 sub validSession {
@@ -52,19 +63,25 @@ sub handler {
             $self->logger->error("Bad JSON content in cookie pdata");
             $req->pdata( {} );
         }
+
+        # Avoid fatal errors when using old keepPdata format
+        if ( $req->pdata->{keepPdata}
+            and not( ref $req->pdata->{keepPdata} eq "ARRAY" ) )
+        {
+            $req->pdata->{keepPdata} = [];
+        }
     }
     my $res = $self->Lemonldap::NG::Common::PSGI::Router::handler($req);
 
     # Avoid permanent loop 'Portal <-> _url' if pdata cookie is not removed
-    my $url64 = encode_base64( $req->userData->{_url}, '' )
-      if $req->userData->{_url};
-    if (    $url64
+    if (    $req->userData->{_url}
         and !$req->pdata->{keepPdata}
         and $req->userData->{_session_id}
         and $req->{env}->{HTTP_COOKIE}
-        and $req->{env}->{HTTP_COOKIE} =~ /$url64/ )
+        and $req->{env}->{HTTP_COOKIE} eq
+        encode_base64( $req->userData->{_url}, '' ) )
     {
-        $self->logger->debug("Force cleaning pdata");
+        $self->logger->info("Force cleaning pdata");
         $self->logger->warn("pdata cookie domain must be set")
           unless ( $self->conf->{pdataDomain} );
         $req->pdata( {} );
@@ -73,7 +90,8 @@ sub handler {
     # Save pdata
     if ( $sp or %{ $req->pdata } ) {
         my %v = (
-            name => $self->conf->{cookieName} . 'pdata',
+            name   => $self->conf->{cookieName} . 'pdata',
+            secure => $self->conf->{securedCookie},
             (
                 %{ $req->pdata }
                 ? ( value => uri_escape( JSON::to_json( $req->pdata ) ) )
@@ -177,10 +195,8 @@ sub refresh {
     $req->steps( [
             'getUser',
             @{ $self->betweenAuthAndData },
-            'setAuthSessionInfo',
             'setSessionInfo',
-            'setMacros',
-            'setGroups',
+            $self->groupsAndMacros,
             'setLocalGroups',
             sub {
                 $req->sessionInfo->{$_} = $data{$_} foreach ( keys %data );
@@ -245,11 +261,25 @@ sub do {
     if ( !$self->conf->{noAjaxHook} and $req->wantJSON ) {
         $self->logger->debug('Processing to JSON response');
         if ( ( $err > 0 and !$req->id ) or $err eq PE_SESSIONNOTGRANTED ) {
-            return [
-                401,
-                [ 'WWW-Authenticate' => "SSO " . $self->conf->{portal} ],
-                [qq'{"result":0,"error":$err}']
-            ];
+            my $json = { result => 0, error => $err };
+            if ( $req->wantErrorRender ) {
+                $json->{html} = $self->loadTemplate(
+                    $req,
+                    'errormsg',
+                    params => {
+                        AUTH_ERROR      => $err,
+                        AUTH_ERROR_TYPE => $req->error_type,
+                    }
+                );
+            }
+            return $self->sendJSONresponse(
+                $req, $json,
+                code    => 401,
+                headers => [
+                    'WWW-Authenticate' => "SSO " . $self->conf->{portal},
+                    "Content-Type"     => "application/javascript"
+                ],
+            );
         }
         elsif ( $err > 0 and $err != PE_PASSWORD_OK and $err != PE_LOGOUT_OK ) {
             return $self->sendJSONresponse(
@@ -346,6 +376,13 @@ sub autoRedirect {
             $req->data->{redirectFormMethod} = "get";
         }
         else {
+            if (    $req->{pdata}->{_url}
+                and $req->{pdata}->{_url} eq encode_base64( $req->{urldc}, '' )
+              )
+            {
+                $self->logger->info("Force cleaning pdata");
+                delete $req->{pdata}->{_url};
+            }
             return [ 302, [ Location => $req->{urldc}, $req->spliceHdrs ], [] ];
         }
     }
@@ -429,7 +466,7 @@ sub getPersistentSession {
     return unless ( defined $uid and !$self->conf->{disablePersistentStorage} );
 
     # Compute persistent identifier
-    my $pid = $self->_md5hash($uid);
+    my $pid = getPSessionID($uid);
 
     $info->{_session_uid} = $uid;
 
@@ -494,7 +531,7 @@ sub updatePersistentSession {
 
     if ( $persistentSession->error ) {
         $self->logger->error(
-            "Cannot update persistent session " . $self->_md5hash($uid) );
+            "Cannot update persistent session " . getPSessionID($uid) );
         $self->logger->error( $persistentSession->error );
     }
 }
@@ -514,13 +551,14 @@ sub updateSession {
     return () unless ( ref $infos eq 'HASH' and %$infos );
 
     # Recover session ID unless given
-    $id ||= $req->{id} || $req->userData->{_session_id};
+    $id ||= $req->id || $req->userData->{_session_id};
 
     if ($id) {
 
         # Update sessionInfo data
         ## sessionInfo updated if $id defined : quite strange !!
         ## See https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues/430
+        $self->logger->debug("Update session $id");
         foreach ( keys %$infos ) {
             $self->logger->debug("Update sessionInfo $_");
             $self->_dump( $infos->{$_} );
@@ -587,7 +625,7 @@ sub _deleteSession {
             name    => $self->conf->{cookieName},
             value   => 0,
             domain  => $self->conf->{domain},
-            secure  => 0,
+            secure  => $self->conf->{securedCookie},
             expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
         )
     ) unless ($preserveCookie);
@@ -602,12 +640,6 @@ sub _deleteSession {
     return $session->error ? 0 : 1;
 }
 
-# Return md5(s)
-sub _md5hash {
-    my ( $self, $s ) = @_;
-    return substr( Digest::MD5::md5_hex($s), 0, 32 );
-}
-
 # Check if an URL's domain name is declared in LL::NG config or is declared as
 # trusted domain
 sub isTrustedUrl {
@@ -641,7 +673,7 @@ sub autoPost {
 
     # Display info before redirecting
     if ( $req->info() ) {
-        $req->{infoFormMethod} = $req->param('method') || "post";
+        $req->data->{infoFormMethod} = $req->param('method') || "post";
         return PE_INFO;
     }
 
@@ -753,7 +785,9 @@ sub cookie {
     $h{HttpOnly} //= $self->conf->{httpOnly};
     $h{max_age}  //= $self->conf->{cookieExpiration}
       if ( $self->conf->{cookieExpiration} );
-    foreach (qw(domain path expires max_age HttpOnly)) {
+    $h{SameSite} ||= $self->cookieSameSite;
+
+    foreach (qw(domain path expires max_age HttpOnly SameSite)) {
         my $f = $_;
         $f =~ s/_/-/g;
         push @res, "$f=$h{$_}" if ( $h{$_} );
@@ -863,10 +897,18 @@ sub sendHtml {
     $csp .= ';';
 
     # Deny using portal in frame except if it is required
-    unless ( $req->frame or $self->conf->{portalAntiFrame} == 0 ) {
+    unless ( $req->frame
+        or $self->conf->{portalAntiFrame} == 0
+        or $self->conf->{cspFrameAncestors} )
+    {
         push @{ $res->[1] }, 'X-Frame-Options' => 'DENY';
         $csp .= "frame-ancestors 'none';";
     }
+    if ( $self->conf->{cspFrameAncestors} ) {
+        push @{ $res->[1] }, 'X-Frame-Options' => 'ALLOW-FROM '
+          . "$self->{conf}->{cspFrameAncestors};";
+        $csp .= "frame-ancestors $self->{conf}->{cspFrameAncestors};";
+    }
 
     # Check if frames need to be embedded
     my @url;
@@ -969,6 +1011,7 @@ sub tplParams {
         PORTAL_URL => $self->conf->{portal},
         SKIN_PATH  => $portalPath . "skins",
         SKIN_BG    => $self->conf->{portalSkinBackground},
+        CUSTOM_CSS => $self->conf->{portalCustomCss},
         ( $self->customParameters ? ( %{ $self->customParameters } ) : () ),
         %templateParams
     );
@@ -1065,7 +1108,30 @@ sub corsPreflight {
 sub sendJSONresponse {
     my ( $self, $req, $j, %args ) = @_;
     my $res = Lemonldap::NG::Common::PSGI::sendJSONresponse(@_);
-    if ( $self->conf->{corsEnabled} ) {
+
+    # Handle caching
+    if ( $args{ttl} and $args{ttl} =~ /^\d+$/ ) {
+        push @{ $res->[1] }, 'Cache-Control' => 'public, max-age=' . $args{ttl};
+    }
+    else {
+        push @{ $res->[1] },
+          'Cache-Control' => 'no-cache, no-store, must-revalidate',
+          'Pragma'        => 'no-cache',
+          'Expires'       => '0';
+    }
+
+    # If this is a cross-domain request from the portal itself
+    # (Ajax SSL to a different VHost)
+    # we allow CORS
+    if ( $req->origin and index( $self->conf->{portal}, $req->origin ) == 0 ) {
+        $self->logger->debug('AJAX request from portal, allowing CORS');
+        push @{ $res->[1] },
+          "Access-Control-Allow-Origin"      => $req->origin,
+          "Access-Control-Allow-Methods"     => "*",
+          "Access-Control-Allow-Credentials" => "true";
+
+    }
+    elsif ( $self->conf->{corsEnabled} ) {
         my @cors = split /;/, $self->cors;
         push @{ $res->[1] }, @cors;
         $self->logger->debug('Apply following CORS policy :');
