@@ -32,7 +32,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_USERNOTFOUND
 );
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.9';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin',
   'Lemonldap::NG::Portal::Lib::SMTP', 'Lemonldap::NG::Portal::Lib::_tokenRule';
@@ -130,7 +130,6 @@ sub _reset {
 
         # Restore pdata if any
         $req->pdata( $mailSession->data->{_pdata} || {} );
-        $mailSession->remove;
         $searchByMail = 0 unless ( $req->{user} =~ /\@/ );
     }
 
@@ -188,14 +187,14 @@ sub _reset {
 
     # Search user in database
     $req->steps( [
-            'getUser',                  'setSessionInfo',
-            'setMacros',                'setGroups',
-            'setPersistentSessionInfo', 'setLocalGroups'
+            'getUser',                 'setSessionInfo',
+            $self->p->groupsAndMacros, 'setPersistentSessionInfo',
+            'setLocalGroups'
         ]
     );
     if ( my $error = $self->p->process( $req, useMail => $searchByMail ) ) {
         if ( $error == PE_USERNOTFOUND or $error == PE_BADCREDENTIALS ) {
-            $self->userLogger->warn( 'Reset asked for an unvalid user ('
+            $self->userLogger->warn( 'Reset asked for an invalid user ('
                   . $req->param('mail')
                   . ')' );
 
@@ -353,6 +352,8 @@ sub _reset {
         $body =~ s/\$url/$url/g;
         $body =~ s/\$(\w+)/$req->{sessionInfo}->{$1} || ''/ge;
 
+        $self->logger->info("User ".$req->data->{mailAddress}." is trying to reset his/her password");
+
         # Send mail
         unless (
             $self->send_mail(
@@ -409,6 +410,23 @@ sub changePwd {
         return PE_NOTOKEN;
     }
 
+    # Remove the mail token session if mail token is provided
+    my $mailToken = $req->param('mail_token');
+    if ($mailToken) {
+        $self->logger->debug("Token given for password reset: $mailToken");
+
+        # Check if token is valid
+        my $mailSession =
+          $self->p->getApacheSession( $mailToken, kind => "TOKEN" );
+        unless ($mailSession) {
+            $self->userLogger->warn('Bad reset token');
+            return PE_BADMAILTOKEN;
+        }
+
+        $self->logger->debug("Delete token $mailToken");
+        $mailSession->remove;
+    }
+
     # Check if user wants to generate the new password
     if ( $req->param('reset') ) {
         $self->logger->debug(
@@ -525,7 +543,11 @@ sub setSecurity {
 
 sub display {
     my ( $self, $req ) = @_;
+    my $speChars = $self->conf->{passwordPolicySpecialChar};
+    $speChars =~ s/\s+/ /g;
+    $speChars =~ s/(?:^\s|\s$)//g;
     $self->logger->debug( 'Display called with code: ' . $req->error );
+
     my %tplPrm = (
         SKIN_PATH       => $self->conf->{staticPrefix},
         SKIN            => $self->p->getSkin($req),
@@ -541,8 +563,7 @@ sub display {
         STARTMAILTIME   => $req->data->{startMailTime},
         MAILALREADYSENT => $req->data->{mailAlreadySent},
         MAIL            => (
-            $self->p->checkXSSAttack( 'mail', $req->{user} )
-            ? ''
+              $self->p->checkXSSAttack( 'mail', $req->{user} ) ? ''
             : $req->{user}
         ),
         DISPLAY_FORM            => 0,
@@ -555,6 +576,12 @@ sub display {
         PPOLICY_MINLOWER        => $self->conf->{passwordPolicyMinLower},
         PPOLICY_MINUPPER        => $self->conf->{passwordPolicyMinUpper},
         PPOLICY_MINDIGIT        => $self->conf->{passwordPolicyMinDigit},
+        PPOLICY_ALLOWEDSPECHAR  => $speChars,
+        (
+            $speChars
+            ? ( PPOLICY_MINSPECHAR => $self->conf->{passwordPolicyMinSpeChar} )
+            : ()
+        ),
         DISPLAY_GENERATE_PASSWORD =>
           $self->conf->{portalDisplayGeneratePassword},
     );
