@@ -15,7 +15,7 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_FORMEMPTY);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.6';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin',
   'Lemonldap::NG::Common::Conf::AccessLib';
@@ -43,7 +43,7 @@ has exportedAttr => (
                 'authenticationLevel', 'groups',
                 'ipAddr',              '_startTime',
                 '_utime',              '_lastSeen',
-                '_session_id',
+                '_session_id',         '_session_kind',
             );
             if ( my $exportedAttr = $conf->{exportedAttr} ) {
                 $exportedAttr =~ s/^\s*\+\s+//;
@@ -78,7 +78,7 @@ has wsdl => (
 
         my $attrList = join "\n", map {
             "<element name='$_' type='xsd:string' nillable='true'></element>"
-        } $self->exportedAttr;
+        } @{ $self->exportedAttr };
         my $resp = join( '', <DATA> );
         close DATA;
         $resp =~ s/\$cookieList/$cookieList/g;
@@ -272,7 +272,7 @@ sub getAttributes {
     my ( $self, $req, $id ) = @_;
     die 'id is required' unless ($id);
 
-    my $session = $self->p->getApacheSession($id);
+    my $session = $self->p->getApacheSession( $id, kind => '' );
 
     my @tmp = ();
     unless ($session) {
@@ -384,7 +384,7 @@ sub deleteSession {
     my ( $self, $req, $id ) = @_;
     die('id parameter is required') unless ($id);
 
-    my $session = $self->p->getApacheSession($id);
+    my $session = $self->p->getApacheSession( $id, kind => '' );
 
     return 0 unless ($session);
 
